@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,17 +13,18 @@ the Free Software Foundation, either version 3 of the License, or
 For more see the file 'readme/COPYING' for copying permission.
 """
 
+import sys
+
 # Dummy check for missing module(s).
 try:
   __import__("src.utils.version")
   from src.utils import version
   version.python_version()
 
-except ImportError:
-  err_msg = "Wrong installation detected (missing modules). "
-  err_msg = "Visit 'https://github.com/commixproject/commix/' for further details. \n"
-  print(settings.print_critical_msg(err_msg))
-  raise SystemExit()
+except ImportError as ex:
+  err_msg = "Wrong installation detected (i.e \"" + str(ex) + "\"). "
+  err_msg += "Visit 'https://github.com/commixproject/commix/' for further details."
+  sys.exit(err_msg)
 
 # Main
 def main():
@@ -41,7 +42,7 @@ if __name__ == '__main__':
     raise SystemExit()
   except IndentationError as err_msg:
     from src.utils import settings
-    print(settings.print_critical_msg(err_msg) + ".\n")
+    settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
     raise SystemExit()
   except:
     from src.utils import common
