@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ def http_response_content(content):
     content = content.decode(settings.DEFAULT_CODEC)
   if settings.VERBOSITY_LEVEL >= 4:
     content = checks.remove_empty_lines(content)
-    print(settings.print_http_response_content(content))
+    settings.print_data_to_stdout(settings.print_http_response_content(content))
   if menu.options.traffic_file:
     logs.log_traffic(content)
     logs.log_traffic("\n\n" + "#" * 77 + "\n\n")
@@ -63,7 +63,7 @@ def http_response(headers, code):
   for header in response_http_headers:
     if len(header) > 1:
       if settings.VERBOSITY_LEVEL >= 3:
-        print(settings.print_traffic(header))
+        settings.print_data_to_stdout(settings.print_traffic(header))
       if menu.options.traffic_file:
         logs.log_traffic("\n" + header)
   if menu.options.traffic_file:
@@ -73,17 +73,22 @@ def http_response(headers, code):
 Print HTTP response headers / Body.
 """
 def print_http_response(response_headers, code, page):
+  if int(code) in settings.ABORT_CODE:
+    err_msg = "Aborting due to detected HTTP code '" + str(code) + "'. "
+    settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
+    raise SystemExit()
+
   if settings.VERBOSITY_LEVEL >= 3 or menu.options.traffic_file:
     if settings.VERBOSITY_LEVEL >= 3:
       resp_msg = "HTTP response [" + settings.print_request_num(settings.TOTAL_OF_REQUESTS) + "] (" + str(code) + "):"
-      print(settings.print_response_msg(resp_msg))
+      settings.print_data_to_stdout(settings.print_response_msg(resp_msg))
     if menu.options.traffic_file:
       resp_msg = "HTTP response [#" + str(settings.TOTAL_OF_REQUESTS) + "] (" + str(code) + "):"
       logs.log_traffic("\n" + resp_msg)
     http_response(response_headers, code)
   if settings.VERBOSITY_LEVEL >= 4 or menu.options.traffic_file:
     if settings.VERBOSITY_LEVEL >= 4:
-      print(settings.SINGLE_WHITESPACE)
+      settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
     try:
       http_response_content(page)
     except AttributeError:
@@ -107,13 +112,13 @@ def check_http_traffic(request):
       unique_request_http_headers = []
       [unique_request_http_headers.append(item) for item in request_http_headers if item not in unique_request_http_headers]
       request_http_headers = [x for x in unique_request_http_headers if x]
-      if menu.options.data and \
+      if settings.USER_DEFINED_POST_DATA and \
          len(request_http_headers) == 1 and \
          settings.VERBOSITY_LEVEL >= 2:
-        print(settings.SINGLE_WHITESPACE)
+        settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
       for header in request_http_headers:
         if settings.VERBOSITY_LEVEL >= 2:
-          print(settings.print_traffic(header))
+          settings.print_data_to_stdout(settings.print_traffic(header))
         if menu.options.traffic_file:
           logs.log_traffic("\n" + header)
       http_client.send(self, req)
@@ -127,7 +132,7 @@ def check_http_traffic(request):
       if settings.VERBOSITY_LEVEL >= 2 or menu.options.traffic_file:
         if settings.VERBOSITY_LEVEL >= 2:
           req_msg = "HTTP request [" + settings.print_request_num(settings.TOTAL_OF_REQUESTS) + "]:"
-          print(settings.print_request_msg(req_msg))
+          settings.print_data_to_stdout(settings.print_request_msg(req_msg))
         if menu.options.traffic_file:
           req_msg = "HTTP request [#" + str(settings.TOTAL_OF_REQUESTS) + "]:"
           logs.log_traffic(req_msg)
@@ -150,17 +155,20 @@ def check_http_traffic(request):
 
   opener = _urllib.request.build_opener(connection_handler())
 
-  if len(settings.HTTP_METHOD) != 0:
-    request.get_method = lambda: settings.HTTP_METHOD
+  # Time limit mechanism.
+  if menu.options.time_limit and (time.time() - settings.START_TIME > menu.options.time_limit):
+    raise SystemExit()
 
   _ = False
   response = False
   unauthorized = False
   while not _ and settings.TOTAL_OF_REQUESTS <= settings.MAX_RETRIES and unauthorized is False:
+    if any((settings.REVERSE_TCP, settings.BIND_TCP)):
+      _ = True
     if settings.MULTI_TARGETS:
       if settings.INIT_TEST == True and len(settings.MULTI_ENCODED_PAYLOAD) != 0:
         settings.MULTI_ENCODED_PAYLOAD = []
-        menu.options.tamper = settings.USER_SUPPLIED_TAMPER
+        menu.options.tamper = settings.USER_APPLIED_TAMPER
     try:
       response = opener.open(request, timeout=settings.TIMEOUT)
       _ = True
@@ -174,9 +182,9 @@ def check_http_traffic(request):
 
     except ValueError as err:
       if settings.VERBOSITY_LEVEL < 2:
-        print(settings.SINGLE_WHITESPACE)
+        settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
       err_msg = "Invalid target URL has been given."
-      print(settings.print_critical_msg(err_msg))
+      settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
       raise SystemExit()
 
     except AttributeError:
@@ -210,8 +218,7 @@ def check_http_traffic(request):
     # Checks for not declared cookie(s), while server wants to set its own.
     if not menu.options.drop_set_cookie:
       checks.not_declared_cookies(response)
-    if settings.VERBOSITY_LEVEL > 2 or menu.options.traffic_file:
-      print_http_response(response_headers, code, page)
+    print_http_response(response_headers, code, page)
     # Checks regarding a potential CAPTCHA protection mechanism.
     checks.captcha_check(page)
     # Checks regarding a potential browser verification protection mechanism.
@@ -225,12 +232,14 @@ def check_http_traffic(request):
     if not menu.options.drop_set_cookie:
       checks.not_declared_cookies(err)
     try:
+      if err.fp is None:
+        raise AttributeError
       page = checks.page_encoding(err, action="encode")
-    except Exception as ex:
-      page = err.read()
-    if settings.VERBOSITY_LEVEL != 0:
-      print_http_response(err.info(), err.code, page)
-
+    except Exception as e:
+      page = ''
+      
+    print_http_response(err.info(), err.code, page)
+    
     if (not settings.PERFORM_CRACKING and \
     not settings.IS_JSON and \
     not settings.IS_XML and \
@@ -238,7 +247,7 @@ def check_http_traffic(request):
     not str(err.code) == settings.BAD_REQUEST and \
     not settings.CRAWLED_URLS_NUM != 0 and \
     not settings.MULTI_TARGETS) and settings.CRAWLED_SKIPPED_URLS_NUM != 0:
-      print(settings.SINGLE_WHITESPACE)
+      settings.print_data_to_stdout(settings.SINGLE_WHITESPACE)
     # Check for 3xx, 4xx, 5xx HTTP error codes.
     if str(err.code).startswith(('3', '4', '5')):
       settings.HTTP_ERROR_CODES_SUM.append(err.code)
@@ -253,7 +262,7 @@ def check_http_traffic(request):
       else:
         err_msg = error_msg
 
-      print(settings.print_critical_msg(err_msg + ")."))
+      settings.print_data_to_stdout(settings.print_critical_msg(err_msg + ")."))
       raise SystemExit()
 
 """
@@ -262,35 +271,38 @@ Check for added headers.
 def do_check(request):
 
   # Check if defined any Cookie HTTP header.
-  if menu.options.cookie and settings.COOKIE_INJECTION == None:
-    request.add_header(settings.COOKIE, menu.options.cookie)
+  if menu.options.cookie and not settings.COOKIE_INJECTION:
+    request.add_header(settings.COOKIE, checks.remove_tags(menu.options.cookie))
 
   # Check if defined any User-Agent HTTP header.
-  if menu.options.agent and settings.USER_AGENT_INJECTION == None:
-    request.add_header(settings.USER_AGENT, menu.options.agent)
+  if menu.options.agent and not settings.USER_AGENT_INJECTION:
+    request.add_header(settings.USER_AGENT, checks.remove_tags(menu.options.agent))
 
   # Check if defined any Referer HTTP header.
-  if menu.options.referer and settings.REFERER_INJECTION == None:
-    request.add_header(settings.REFERER, menu.options.referer)
+  if menu.options.referer and not settings.REFERER_INJECTION:
+    request.add_header(settings.REFERER, checks.remove_tags(menu.options.referer))
 
   # Check if defined any Host HTTP header.
-  if menu.options.host and settings.HOST_INJECTION == None:
-    request.add_header(settings.HOST, menu.options.host)
+  if menu.options.host and not settings.HOST_INJECTION:
+    request.add_header(settings.HOST, checks.remove_tags(menu.options.host))
 
   if not checks.get_header(request.headers, settings.ACCEPT):
     request.add_header(settings.ACCEPT, settings.ACCEPT_VALUE)
 
+  if not checks.get_header(request.headers, settings.CONTENT_TYPE):
+    request.add_unredirected_header(settings.CONTENT_TYPE, settings.DEFAULT_HTTP_CONTENT_TYPE_VALUE)
+
   # The MIME media type for JSON.
   if menu.options.data and not (menu.options.requestfile or menu.options.logfile):
     if re.search(settings.JSON_RECOGNITION_REGEX, menu.options.data) or \
        re.search(settings.JSON_LIKE_RECOGNITION_REGEX, menu.options.data):
-      request.add_header(settings.CONTENT_TYPE, settings.HTTP_CONTENT_TYPE_JSON_HEADER_VALUE)
+      request.add_unredirected_header(settings.CONTENT_TYPE, settings.HTTP_CONTENT_TYPE_JSON_HEADER_VALUE)
     elif re.search(settings.XML_RECOGNITION_REGEX, menu.options.data):
-      request.add_header(settings.CONTENT_TYPE, settings.HTTP_CONTENT_TYPE_XML_HEADER_VALUE)
+      request.add_unredirected_header(settings.CONTENT_TYPE, settings.HTTP_CONTENT_TYPE_XML_HEADER_VALUE)
 
   # Default value for "Accept-Encoding" HTTP header
   if not (menu.options.requestfile or menu.options.logfile):
-    request.add_header('Accept-Encoding', settings.HTTP_ACCEPT_ENCODING_HEADER_VALUE)
+    request.add_header(settings.ACCEPT_ENCODING, settings.HTTP_ACCEPT_ENCODING_HEADER_VALUE)
 
   # Appends a fake HTTP header 'X-Forwarded-For' (and alike)
   if settings.TAMPER_SCRIPTS["xforwardedfor"]:
@@ -302,7 +314,7 @@ def do_check(request):
   if menu.options.auth_cred and menu.options.auth_type:
     if menu.options.auth_type.lower() not in (settings.AUTH_TYPE.BASIC, settings.AUTH_TYPE.DIGEST, settings.AUTH_TYPE.BEARER):
       err_msg = "HTTP authentication type value must be Basic, Digest or Bearer."
-      print(settings.print_critical_msg(err_msg))
+      settings.print_data_to_stdout(settings.print_critical_msg(err_msg))
       raise SystemExit()
     if menu.options.auth_type.lower() == settings.AUTH_TYPE.BEARER:
       request.add_header(settings.AUTHORIZATION, "Bearer " + menu.options.auth_cred.strip())
@@ -317,8 +329,8 @@ def do_check(request):
         except _urllib.error.HTTPError as e:
           try:
             authline = e.headers.get('www-authenticate', '')
-            authobj = re.match('''(\w*)\s+realm=(.*),''',authline).groups()
-            realm = authobj[1].split(',')[0].replace("\"","")
+            authobj = re.match(r'''(\w*)\s+realm=(.*),''',authline).groups()
+            realm = authobj[1].split(',')[0].replace("\"", "")
             user_pass_pair = menu.options.auth_cred.split(":")
             username = user_pass_pair[0]
             password = user_pass_pair[1]
@@ -336,33 +348,32 @@ def do_check(request):
     pass
 
   # Check if defined any extra HTTP headers.
-  if menu.options.headers or menu.options.header or len(settings.RAW_HTTP_HEADERS) >= 1:
-    if len(settings.RAW_HTTP_HEADERS) >= 1:
+  if settings.EXTRA_HTTP_HEADERS or settings.RAW_HTTP_HEADERS:
+    if settings.RAW_HTTP_HEADERS:
       menu.options.headers = settings.RAW_HTTP_HEADERS
-    # Do replacement with the 'INJECT_HERE' tag, if the wildcard char is provided.
+    # Do replacement with the 'INJECT_HERE' tag, if the custom injection marker character is provided.
     if menu.options.headers:
-      menu.options.headers = checks.wildcard_character(menu.options.headers)
-      extra_headers = menu.options.headers
-    else:
-      menu.options.header = checks.wildcard_character(menu.options.header)
-      extra_headers = menu.options.header
+      extra_headers = checks.process_custom_injection_data(menu.options.headers)
+    elif menu.options.header:
+      extra_headers = checks.process_custom_injection_data(menu.options.header)
 
     extra_headers = extra_headers.replace(":",": ")
-
     if ": //" in extra_headers:
       extra_headers = extra_headers.replace(": //" ,"://")
 
     if "\\n" in extra_headers:
       extra_headers = extra_headers.split("\\n")
       # Remove empty strings and "Content-Length"
-      extra_headers = [x for x in extra_headers if "Content-Length" not in x]
+      extra_headers = [x for x in extra_headers if settings.CONTENT_LENGTH not in x]
     else:
       tmp_extra_header = []
       tmp_extra_header.append(extra_headers)
       extra_headers = tmp_extra_header
 
-    # Remove empty strings
-    extra_headers = [x for x in extra_headers if x]
+    # Remove empty strings and/or duplicates
+    _ = [x for x in extra_headers if x]
+    extra_headers = (list(dict.fromkeys(_)))
+
     if menu.options.data:
       # The MIME media type for JSON.
       if re.search(settings.JSON_RECOGNITION_REGEX, menu.options.data) or \
@@ -372,8 +383,8 @@ def do_check(request):
       elif re.search(settings.XML_RECOGNITION_REGEX, menu.options.data):
          if settings.CONTENT_TYPE not in str(extra_headers):
           request.add_header(settings.CONTENT_TYPE, settings.HTTP_CONTENT_TYPE_XML_HEADER_VALUE)
-    if "Accept-Encoding" not in str(extra_headers):
-      request.add_header('Accept-Encoding', settings.HTTP_ACCEPT_ENCODING_HEADER_VALUE)
+    if settings.ACCEPT_ENCODING not in str(extra_headers):
+      request.add_header(settings.ACCEPT_ENCODING, settings.HTTP_ACCEPT_ENCODING_HEADER_VALUE)
 
     for extra_header in extra_headers:
       try:
@@ -384,12 +395,24 @@ def do_check(request):
         http_header_value = extra_header.split(':', 1)[1]
         http_header_value = ''.join(http_header_value).strip().replace(": ",":")
         # Check if it is a custom header injection.
-        if settings.CUSTOM_HEADER_INJECTION == False and http_header_name in settings.TEST_PARAMETER:
-          settings.CUSTOM_HEADER_INJECTION = True
-          settings.CUSTOM_HEADER_NAME = http_header_name
-          settings.CUSTOM_HEADER_VALUE = http_header_value
-        # Add HTTP Header name / value to the HTTP request
-        if http_header_name not in [settings.HOST, settings.USER_AGENT, settings.REFERER, settings.COOKIE] and settings.CUSTOM_HEADER_INJECTION == False:
+        if http_header_name not in [settings.ACCEPT, settings.HOST, settings.USER_AGENT, settings.REFERER, settings.COOKIE]:
+          if not settings.CUSTOM_HEADER_INJECTION:
+            if settings.CUSTOM_INJECTION_MARKER_CHAR in http_header_value:
+              settings.CUSTOM_HEADER_CHECK = http_header_name
+              # settings.CUSTOM_INJECTION_MARKER = True
+              
+            if http_header_name in settings.TESTABLE_PARAMETERS_LIST or settings.INJECT_TAG in http_header_value or settings.ASTERISK_MARKER in http_header_value:
+              settings.INJECTION_MARKER_LOCATION.CUSTOM_HTTP_HEADERS = True
+              settings.CUSTOM_HEADER_CHECK = http_header_name
+              if len(http_header_name) != 0 and \
+                http_header_name + ": " + http_header_value not in [settings.ACCEPT, settings.HOST, settings.USER_AGENT, settings.REFERER, settings.COOKIE] and \
+                http_header_name + ": " + http_header_value not in settings.CUSTOM_HEADERS_NAMES:
+                settings.CUSTOM_INJECTION_MARKER_PARAMETERS_LIST.append(http_header_name) if http_header_name not in settings.CUSTOM_INJECTION_MARKER_PARAMETERS_LIST else settings.CUSTOM_INJECTION_MARKER_PARAMETERS_LIST
+                settings.CUSTOM_HEADERS_NAMES.append(http_header_name + ": " + http_header_value)
+              http_header_value = checks.remove_tags(http_header_value)
+              request.add_header(http_header_name, http_header_value)
+              
+        if http_header_name not in [settings.HOST, settings.USER_AGENT, settings.REFERER, settings.COOKIE, settings.CUSTOM_HEADER_NAME]:
           request.add_header(http_header_name, http_header_value)
       except:
         pass
