@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -29,7 +29,16 @@ Use the defined HTTP Proxy
 """
 def use_proxy(request):
   try:
-    request.set_proxy(menu.options.proxy, settings.PROXY_SCHEME)
+    if menu.options.ignore_proxy:
+      proxy = _urllib.request.ProxyHandler({})
+      opener = _urllib.request.build_opener(proxy)
+      _urllib.request.install_opener(opener)
+    elif menu.options.tor:
+      proxy = _urllib.request.ProxyHandler({settings.TOR_HTTP_PROXY_SCHEME:menu.options.proxy})
+      opener = _urllib.request.build_opener(proxy)
+      _urllib.request.install_opener(opener)
+    else:
+      request.set_proxy(menu.options.proxy, settings.SCHEME)
     return _urllib.request.urlopen(request, timeout=settings.TIMEOUT)
   except Exception as err_msg:
     return requests.request_failed(err_msg)
@@ -40,6 +49,6 @@ def use_proxy(request):
 def do_check():
   if settings.VERBOSITY_LEVEL != 0:
     debug_msg = "Setting the HTTP proxy for all HTTP requests. "
-    print(settings.print_debug_msg(debug_msg))
+    settings.print_data_to_stdout(settings.print_debug_msg(debug_msg))
 
 # eof
\ No newline at end of file
