@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@ from src.utils import menu
 from src.utils import settings
 
 """
-About: Adds back slashes (\) between the characters of the generated payloads.
+About: Adds back slashes ("\") between the characters of the generated payloads.
 Notes: This tamper script works against Unix-like target(s).
 """
 
@@ -36,7 +36,7 @@ def tamper(payload):
     for word in settings.IGNORE_TAMPER_TRANSFORMATION:
       _ = obf_char.join(word[i:i+1] for i in range(-1, len(word), 1))
       if _ in payload:
-        payload = payload.replace(_,_.replace(obf_char,""))
+        payload = payload.replace(_,_.replace(obf_char, ""))
     return payload
 
   if settings.TARGET_OS != settings.OS.WINDOWS:
