@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -13,7 +13,6 @@ the Free Software Foundation, either version 3 of the License, or
 For more see the file 'readme/COPYING' for copying permission.
 """
 
-import sys
 from src.utils import settings
 
 """
@@ -23,7 +22,13 @@ Notes: This tamper script works against Unix-like target(s).
 
 __tamper__ = "backticks"
 
-settings.TAMPER_SCRIPTS[__tamper__] = True
-settings.USE_BACKTICKS = True
+if not settings.TAMPER_SCRIPTS[__tamper__]:
+  settings.TAMPER_SCRIPTS[__tamper__] = True
+
+def tamper(payload):
+  settings.TAMPER_SCRIPTS[__tamper__] = True
+  settings.USE_BACKTICKS = True
+  payload = payload.replace("$((", "`expr" + settings.WHITESPACES[0]).replace("))", "`")
+  return payload
 
 # eof
\ No newline at end of file
