@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,7 +23,10 @@ Notes: Useful to bypass very weak and bespoke web application firewalls that has
 
 __tamper__ = "multiplespaces"
 
-settings.TAMPER_SCRIPTS[__tamper__] = True
-settings.WHITESPACES[0] = settings.WHITESPACES[0] * random.randrange(2, 8)
-
+def tamper(payload):
+  if not (settings.TAMPER_SCRIPTS[__tamper__]):
+    settings.TAMPER_SCRIPTS[__tamper__] = True
+    for i in range(0, len(settings.WHITESPACES)):
+      settings.WHITESPACES[i] = settings.WHITESPACES[i] * random.randrange(3, 8)
+  return payload
 # eof
\ No newline at end of file
