@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -14,6 +14,7 @@ For more see the file 'readme/COPYING' for copying permission.
 """
 
 from src.utils import settings
+from src.thirdparty.six.moves import urllib as _urllib
 
 """
 About: Replaces space character ('%20') with horizontal tab ('%09')
@@ -28,10 +29,11 @@ if not settings.TAMPER_SCRIPTS[__tamper__]:
 
 def tamper(payload):
   settings.TAMPER_SCRIPTS[__tamper__] = True
-  if settings.WHITESPACES[0] == "%20":
-    settings.WHITESPACES[0] = space2htab
-  elif space2htab not in settings.WHITESPACES:
-    settings.WHITESPACES.append(space2htab)
+  if len(settings.WHITESPACES) != 0:
+    if settings.WHITESPACES[0] == _urllib.parse.quote(settings.SINGLE_WHITESPACE):
+      settings.WHITESPACES[0] = space2htab
+    elif space2htab not in settings.WHITESPACES:
+      settings.WHITESPACES.append(space2htab)
   return payload
 
 # eof
\ No newline at end of file
