@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -30,16 +30,16 @@ if settings.IS_WINDOWS:
 The commix's banner.
 """
 def banner():
-  print("""                                      __
+  settings.print_data_to_stdout(r"""                                      __
    ___   ___     ___ ___     ___ ___ /\_\   __  _
- /`___\ / __`\ /' __` __`\ /' __` __`\/\ \ /\ \/'\  """ + settings.COLOR_VERSION + """
-/\ \__//\ \/\ \/\ \/\ \/\ \/\ \/\ \/\ \ \ \\\/>  </
-\ \____\ \____/\ \_\ \_\ \_\ \_\ \_\ \_\ \_\\/\_/\\_\\ """ + Fore.GREY + Style.UNDERLINE + settings.APPLICATION_URL + Style.RESET_ALL + """
- \/____/\/___/  \/_/\/_/\/_/\/_/\/_/\/_/\/_/\//\/_/ (""" + Fore.LIGHTRED_EX + settings.APPLICATION_TWITTER + Style.RESET_ALL + """)
+ /`___\ / __`\ /' __` __`\ /' __` __`\/\ \ /\ \/'\  """ + settings.COLOR_VERSION + r"""
+/\ \__//\ \/\ \/\ \/\ \/\ \/\ \/\ \/\ \ \ \\/>  </
+\ \____\ \____/\ \_\ \_\ \_\ \_\ \_\ \_\ \_\/\_/\_\ """ + Fore.GREY + Style.UNDERLINE + settings.APPLICATION_URL + Style.RESET_ALL + r"""
+ \/____/\/___/  \/_/\/_/\/_/\/_/\/_/\/_/\/_/\//\/_/ (""" + Fore.LIGHTRED_EX + settings.APPLICATION_X_ACCOUNT + Style.RESET_ALL + """)
 
 +--
 """ + Style.BRIGHT + settings.DESCRIPTION_FULL + Style.RESET_ALL + """
-Copyright © """ + settings.YEAR + """ """ + settings.AUTHOR + Style.RESET_ALL + """ (""" + Fore.LIGHTRED_EX  + settings.AUTHOR_TWITTER + Style.RESET_ALL + """)
+Copyright © """ + settings.YEAR + """ """ + settings.AUTHOR + Style.RESET_ALL + """ (""" + Fore.LIGHTRED_EX  + settings.AUTHOR_X_ACCOUNT + Style.RESET_ALL + """)
 +--
 """)
 
@@ -103,6 +103,11 @@ general.add_option("-t",
                 default=None,
                 help="Log all HTTP traffic into a textual file.")
 
+general.add_option("--time-limit",
+                dest="time_limit",
+                type=float,
+                help="Run with a time limit in seconds (e.g. 3600).")
+
 general.add_option("--batch",
                 action="store_true",
                 dest="batch",
@@ -125,7 +130,7 @@ general.add_option("--charset",
                 action="store",
                 dest="charset",
                 default=None,
-                help="Time-related injection charset (e.g. \"0123456789abcdef\")")
+                help="Time-related injection charset (e.g. '0123456789abcdef').")
 
 general.add_option("--check-internet",
                 action="store_true",
@@ -134,7 +139,7 @@ general.add_option("--check-internet",
 
 general.add_option("--answers",
                 dest="answers",
-                help="Set predefined answers (e.g. \"quit=N,follow=N\")")
+                help="Set predefined answers (e.g. 'quit=N,follow=N').")
 
 # Target options
 target = OptionGroup(parser, Style.BRIGHT + Style.UNDERLINE + "Target" + Style.RESET_ALL,
@@ -172,7 +177,7 @@ target.add_option("--crawl",
 target.add_option("--crawl-exclude",
                 dest="crawl_exclude",
                 default=None,
-                help="Regexp to exclude pages from crawling (e.g. \"logout\").")
+                help="Regexp to exclude pages from crawling (e.g. 'logout').")
 
 target.add_option("-x",
                 dest="sitemap_url",
@@ -180,7 +185,7 @@ target.add_option("-x",
 
 target.add_option("--method",
                 dest="method",
-                help="Force usage of given HTTP method (e.g. PUT)")
+                help="Force usage of given HTTP method (e.g. 'PUT').")
 
 # Request options
 request = OptionGroup(parser,  Style.BRIGHT + Style.UNDERLINE + "Request" + Style.RESET_ALL,
@@ -284,11 +289,17 @@ request.add_option("--auth-cred",
                 dest="auth_cred",
                 help="HTTP authentication credentials (e.g. 'admin:admin').")
 
+request.add_option("--abort-code",
+                action="store",
+                dest="abort_code",
+                default=False,
+                help="Abort on (problematic) HTTP error code(s) (e.g. 401).")
+
 request.add_option("--ignore-code",
                 action="store",
                 dest="ignore_code",
                 default=False,
-                help="Ignore (problematic) HTTP error code (e.g. 401).")
+                help="Ignore (problematic) HTTP error code(s) (e.g. 401).")
 
 request.add_option("--force-ssl",
                 action="store_true",
@@ -296,6 +307,12 @@ request.add_option("--force-ssl",
                 default=False,
                 help="Force usage of SSL/HTTPS.")
 
+request.add_option("--ignore-proxy",
+                action="store_true",
+                dest="ignore_proxy",
+                default=False,
+                help="Ignore system default proxy settings.")
+
 request.add_option("--ignore-redirects",
                 action="store_true",
                 dest="ignore_redirects",
@@ -469,11 +486,11 @@ injection.add_option("--delay",
                 help="Seconds to delay between each HTTP request.")
 
 injection.add_option("--time-sec",
-                default=1,
+                default=0,
                 action="store",
                 type="int",
                 dest="timesec",
-                help="Seconds to delay the OS response (Default: 1).")
+                help="Seconds to delay the OS response.")
 
 injection.add_option("--tmp-path",
                 action="store",
@@ -522,9 +539,9 @@ detection = OptionGroup(parser, Style.BRIGHT + Style.UNDERLINE + "Detection" + S
                         "used to customize the detection phase.")
 
 detection.add_option("--level",
-                dest="level",
                 type="int",
-                default=1,
+                dest="level",
+                default=False,
                 help="Level of tests to perform (1-3, Default: " + str(settings.DEFAULT_INJECTION_LEVEL) + ").")
 
 detection.add_option("--skip-calc",
@@ -546,6 +563,12 @@ detection.add_option("--failed-tries",
                 default=len(settings.SEPARATORS_LVL1) - 1,
                 help="Set a number of failed injection tries, in file-based technique.")
 
+detection.add_option("--smart",
+                action="store_true",
+                dest="smart",
+                default=False,
+                help="Perform thorough tests only if positive heuristic(s).")
+
 # Miscellaneous options
 misc = OptionGroup(parser, Style.BRIGHT + Style.UNDERLINE + "Miscellaneous" + Style.RESET_ALL)
 
@@ -583,7 +606,7 @@ misc.add_option("--skip-waf",
                 action="store_true",
                 dest="skip_waf",
                 default=False,
-                help="Skip heuristic detection of WAF/IPS/IDS protection.")
+                help="Skip heuristic detection of WAF/IPS protection.")
 
 misc.add_option("--mobile",
                 action="store_true",
@@ -595,7 +618,7 @@ misc.add_option("--offline",
                 action="store_true",
                 dest="offline",
                 default=False,
-                help="Work in offline mode.\n")
+                help="Work in offline mode.")
 
 misc.add_option("--wizard",
                 action="store_true",
@@ -615,6 +638,12 @@ parser.add_option("--smoke-test",
                     dest="smoke_test",
                     help=SUPPRESS)
 
+parser.add_option("--ignore-stdin",
+                    action="store_true",
+                    dest="ignore_stdin",
+                    default=False,
+                    help=SUPPRESS)
+
 parser.add_option_group(general)
 parser.add_option_group(target)
 parser.add_option_group(request)
@@ -653,10 +682,10 @@ settings.sys_argv_errors()
 The "os_shell" available options.
 """
 def os_shell_options():
-    print("""""" + Style.BRIGHT + """Available 'os_shell' options:""" + Style.RESET_ALL + """
+    settings.print_data_to_stdout("""""" + Style.BRIGHT + """Available 'os_shell' options:""" + Style.RESET_ALL + """
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """?""" + Style.RESET_ALL + """' to get all the available options.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """back""" + Style.RESET_ALL + """' to move back from the current context.
-""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
+""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' / '""" + Style.BRIGHT + """exit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """reverse_tcp""" + Style.RESET_ALL + """' to get a reverse TCP connection.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """bind_tcp""" + Style.RESET_ALL + """' to set a bind TCP connection.""")
 
@@ -664,11 +693,11 @@ def os_shell_options():
 The "reverse_tcp" available options.
 """
 def reverse_tcp_options():
-    print("""""" + Style.BRIGHT + """Available 'reverse_tcp' options:""" + Style.RESET_ALL + """
+    settings.print_data_to_stdout("""""" + Style.BRIGHT + """Available 'reverse_tcp' options:""" + Style.RESET_ALL + """
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """?""" + Style.RESET_ALL + """' to get all the available options.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """set""" + Style.RESET_ALL + """' to set a context-specific variable to a value.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """back""" + Style.RESET_ALL + """' to move back from the current context.
-""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
+""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' / '""" + Style.BRIGHT + """exit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """os_shell""" + Style.RESET_ALL + """' to get into an operating system command shell.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """bind_tcp""" + Style.RESET_ALL + """' to set a bind TCP connection.""")
 
@@ -676,11 +705,11 @@ def reverse_tcp_options():
 The "bind_tcp" available options.
 """
 def bind_tcp_options():
-    print("""""" + Style.BRIGHT + """Available 'bind_tcp' options:""" + Style.RESET_ALL + """
+    settings.print_data_to_stdout("""""" + Style.BRIGHT + """Available 'bind_tcp' options:""" + Style.RESET_ALL + """
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """?""" + Style.RESET_ALL + """' to get all the available options.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """set""" + Style.RESET_ALL + """' to set a context-specific variable to a value.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """back""" + Style.RESET_ALL + """' to move back from the current context.
-""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
+""" + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """quit""" + Style.RESET_ALL + """' / '""" + Style.BRIGHT + """exit""" + Style.RESET_ALL + """' (or use <Ctrl-C>) to quit commix.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """os_shell""" + Style.RESET_ALL + """' to get into an operating system command shell.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """reverse_tcp""" + Style.RESET_ALL + """' to get a reverse TCP connection.""")
 
@@ -688,7 +717,7 @@ def bind_tcp_options():
 The available mobile user agents.
 """
 def mobile_user_agents():
-    print("""""" + Style.BRIGHT + """Available smartphones HTTP User-Agent headers:""" + Style.RESET_ALL + """
+    settings.print_data_to_stdout("""""" + Style.BRIGHT + """Available smartphones HTTP User-Agent headers:""" + Style.RESET_ALL + """
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """1""" + Style.RESET_ALL + """' for BlackBerry Z10.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """2""" + Style.RESET_ALL + """' for Samsung Galaxy S7.
 """ + settings.SUB_CONTENT_SIGN_TYPE + """Type '""" + Style.BRIGHT + """3""" + Style.RESET_ALL + """' for HP iPAQ 6365.
