@@ -3,7 +3,7 @@
 
 """
 This file is part of Commix Project (https://commixproject.com).
-Copyright (c) 2014-2023 Anastasios Stasinopoulos (@ancst).
+Copyright (c) 2014-2024 Anastasios Stasinopoulos (@ancst).
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@ from src.utils import settings
 Show version number and exit.
 """
 def show_version():
-  print(settings.VERSION)
+  settings.print_data_to_stdout(settings.VERSION)
   raise SystemExit()
 
 """
@@ -31,6 +31,7 @@ def python_version():
   if PYTHON_VERSION.split(".")[0] != "3":
     warn_msg = "Deprecated Python version detected: "
     warn_msg += PYTHON_VERSION + ". "
-    warn_msg += "You are advised to use Python version 3."
-    print("\n" + settings.print_bold_warning_msg(warn_msg))
-    #raise SystemExit()
+    warn_msg += "You are advised to re-run with Python 3."
+    settings.print_data_to_stdout(settings.print_bold_warning_msg(warn_msg))
+
+# eof
\ No newline at end of file
